function strainz_in = make_example_in_structure
%
% v1, 10/16/08
%
% Example function to create test input for strainz16 or strainz17. 

% GOALS OF THIS FUNCTION

% This function is meant to be a simple example of creating the strainz input
% data structure 'in.' This example is intended for users unfamiliar with
% Matlab.  This script is written to be simple and easily understood.  It
% is not written to be efficient, elegant, or compact. 

% This function is not a general purpose input routine.  Its
% parameters are set for the specific example data.  NOTE THAT YOU SHOULD
% READ THE INTRODUCTORY COMMENTS IN strainz*.m FOR A COMPLETE DESCRIPTION
% OF THE INPUT PARAMETERS. 

% WHAT THIS FUNCTION DOES
% This function reads 3-component time series in 14 pre-existing test data
% files named 'stn1.txt', 'stn2.txt', ..., 'stn14.txt'.  This function
% creates a structure named strainz_in, which is the input for strainz*.m.
% This function fills strainz_in with data like the array coordinates.  

% DISCLAIMER: 
%
% Although this program has been used by the USGS, no warranty,
% expressed or implied, is made by the USGS or the United States Government
% or any authors as to the accuracy and functioning of the program and
% related program material nor shall the fact of distribution constitute
% any such warranty, and no responsibility is assumed by the USGS in
% connection therewith.

% COPYRIGHT INFO
% This m-file is a U.S. Government work and is not subject to copyright. It
% may be freely distributed. 


% --------------- begin setting input variables -------------------------

% Set array coordinates relative to some origin. In this example the sixth
% station is the origin, but any point may be selected as the origin.  The
% origin point does not have to coincide with a station. 

array_coords_in_m = [ ... 
%    REL OFFSET (M)
% x1(E) x2(N) x3(up)
-370.2 -304.7 -19.0;
-296.3 -210.6 -22.1;
-191.9 -285.1 -19.2;
  37.0 -180.7 -12.9;
 -20.1  -14.8  -2.1;
   0.0    0.0   0.0;
  22.9  -10.1   3.7;
 181.8   96.9  14.2;
 213.5  234.6  14.6;
 260.1  408.6   3.4;
  99.3  214.6   1.7;
 -71.7  324.3 -12.4;
-131.3  390.0  -3.2;
-195.9  351.6  -1.4  ];

% Convert array coordinates to cm because the example ground motions are
% measured in cm.  If your seismograms are in units of
% miles/year, you must convert the array coordinates to miles here. 
array_coords_in_cm = array_coords_in_m * 100; 

% Determine Na, number of stations in the array
[Na,junk] = size(array_coords_in_cm) ; 

% Specify the subarray (subset of array stations) whose ground motions will
% be fit using strainz*.  This option is useful when you want to analyse
% only a subset of stations.  Here we are analysing only the data from
% stations 1, 2, 6, 7, 8, 9, 10, 11, 13, and 14.
subarray = [ 1 2 6 7 8 9 10 11 13 14];

% Set the standard deviations of the ground motions of all Na stations.  This
% is an estimate of the ground and instrumental noise.  It is allowed to 
% vary from station to station.  The particular values below are
% unrealistic. Stations 2, 5, and 14 have high noise levels. sigmau may be
% input as a scalar (if all stations have the same noise level), a vector
% (if all components of a single station have the same noise level), or a 
% matrix (if the noise level varies from component to component of each
% station).  See comments in strainz*.m for more information. 

q = 0.4; % quiet station 
n = 2.0; % noisy station 
sigmau = [ q n q q q n q q q q q q q n];


% Set P and S wavespeeds of material between the array stations. 
% Note that the P and S speeds must be specified in the same units,
% although those units need not be the same as the array coordinate units,
% because only the Vp/Vs ratio is used.  
Vp = 1.93 % km/s
Vs = 0.326 % km/s

% load the data files.  See the introductory comments in strainz*.m for a
% detailed description of the ground motion input. These test data are not
% real seismograms, just test input.  

% WARNING - there are numerous ways to import ascii data in Matlab, some of
% which are not suitable for Matlab novices.  We use importdata here.
% Users who still think in FORTRAN might prefer to call function fgetl
% (which reads a single formatted line or record) inside a 'while' loop. 

previous_nt = NaN; 

for istn = 1:Na
    
    % this assumes that the data are in the current directory
    filename = ['stn' int2str(istn) '.txt'];
    disp(['Reading data from file ' filename])

    % This puts the 3 components of time series in a nt x 3 matrix
    seisms = importdata(filename); 
    
    % The test data files do not require the following test, but you should
    % use this test when reading your own data.  strainz*.m requires that
    % all time series have the same number of points
    [nt,nc] = size(seisms); 
    if (nt ~= previous_nt) & (istn ~= 1)
        error(['File ' filename ' has differing number of samples'])
    end
    previous_nt = nt; 
    
    % Matlab purists will not like the following because we have not
    % preallocated storage for ts1, for simplicity.
    
    ts1(:,istn) = seisms(:,1); 
    ts2(:,istn) = seisms(:,2); 
    ts3(:,istn) = seisms(:,3); 
    
    % if you do not have a seismogram for station istn, fill ts* with NaNs,
    % like this:  ts1(:,istn) = NaN * ones(nt,1); 
    
end % end loop over all stations in array
    
% Now load all variables into the input structure 'strainz_in'.  We could 
% have loaded the fields of 'strainz_in' directly above, but loading the 
% structure here makes the relation between the structure and its contents 
% obvious.  

strainz_in.vp = Vp;
strainz_in.vs = Vs;
strainz_in.array_coords = array_coords_in_cm;
strainz_in.subarray = subarray;
strainz_in.ts1 = ts1;
strainz_in.ts2 = ts2;
strainz_in.ts3 = ts3;
strainz_in.sigmau = sigmau;

return