function plot_save_strainz_output1(szo)
%
% v2, 10/26/08
%
% GOALS OF THIS ROUTINE
% This is a simple example of plotting and saving the output of strainz*.m,
% intended for the Matlab novice.  It is not intended as a general purpose 
% plotting routine, although it could be modified to be such. 
%
% Input:
%   szo = the output structure created by strainz*.m
%
% Output: Three output files: 
%   1)  Plot is saved in a pdf file named plot_save_strainz_output1.pdf
%   2)  Numerical results are saved in a binary file named
%           plot_save_strainz_output1.mat
%   3)  Numerical results saved in ascii output file ascii_output.txt, 
%           in which row/record i contains:
%           Column 1: t(i), an artificial example time assigned to row i
%           Column 2: maximum shear strain at time t(i)
%           Column 3: dilatation at time t(i)
%           Column 4: rotation (rad) about the x1 axis at time t(i)
%           Column 5: rotation (rad) about the x2 axis at time t(i)
%           Column 6: rotation (rad) about the x3 axis at time t(i)
%           Column 7: mistfit ratio at time t(i)%
%
% NOTE - This function writes the ascii file using the unix/mac line end
% charater, the 'newline' character.  To output the ascii file using
% PC line ends (newline/carriage-return), change the value of the 'lineend'
% variable set below. 
%       
% DISCLAIMER: 
%
% Although this program has been used by the USGS, no warranty,
% expressed or implied, is made by the USGS or the United States Government
% or any authors as to the accuracy and functioning of the program and
% related program material nor shall the fact of distribution constitute
% any such warranty, and no responsibility is assumed by the USGS in
% connection therewith.

% COPYRIGHT INFO
% This m-file is a U.S. Government work and is not subject to copyright. It
% may be freely distributed. 

% ------------- begin active part of code ------------------------

nplots = 6; % dilatation, shear strain, rotation around
% the x1, x2, and x3 axes, and misfit ratio M.

% For illustrative purposes, we will assume that the time series all start
% at time 3.0 s and that the time increment between samples is dt = 0.01 s.
% Note that it is not necessary to know this in order to calculate strains
% and rotations.  We make this assumption to show the Matlab novice how to
% plot a time axis. 
[nt,junk] = size(szo.ts_s); 
tstart = 3.0; % s, example value
dt = 0.01; % s, example value
t = tstart + (1:nt)*dt - dt; 

% ------- plot shear strain ---------------------
subplot(nplots,1,1)
plot(t, szo.ts_s)
xlabel('Values on this time axis are arbitrary  (s)')
ylabel('strain')
title(['Shear strain (unitless)'])

% ------- plot dilatation ---------------------
subplot(nplots,1,2)
plot(t, szo.ts_d)
xlabel('Values on this time axis are arbitrary  (s)')
ylabel('dilatation')
title(['Dilatation (unitless), std dev = ' num2str(szo.sigmad)])

% ------- plot x1 rotation  ---------------------
subplot(nplots,1,3)
plot(t, szo.ts_w1)
xlabel('Values on this time axis are arbitrary  (s)')
ylabel('rotation (rad)')
title(['Rotation around the x1 axis, std dev = ' num2str(szo.sigmaw1)])

% ------- plot x2 rotation  ---------------------
subplot(nplots,1,4)
plot(t, szo.ts_w2)
xlabel('Values on this time axis are arbitrary  (s)')
ylabel('rotation (rad)')
title(['Rotation around the x2 axis, std dev = ' num2str(szo.sigmaw2)])

% ------- plot x3 rotation  ---------------------
subplot(nplots,1,5)
plot(t, szo.ts_w3)
xlabel('Values on this time axis are arbitrary  (s)')
ylabel('rotation (rad)')
title(['Rotation around the x3 axis, std dev = ' num2str(szo.sigmaw3)])

% ------- plot misfit ratio ---------------------
subplot(nplots,1,6)
plot(t, szo.ts_M)
xlabel('Values on this time axis are arbitrary  (s)')
ylabel('M')
title(['Misfit ratio (unitless)'])

% save the plot file to a pdf
print -dpdf plot_save_strainz_output1.pdf
disp('Saving plot in file plot_save_strainz_output1.pdf')

% save the strainz17 output structure to a .mat file
save plot_save_strainz_output1.mat szo
disp('Saving numerical results in file plot_save_strainz_output1.mat')

% save all of the time series in an ascii file
% first insert all the time series as columns in a matrix.  Column 1 is
% time, column 2 is shear strain, 3 is dilatation, etc. 
outputmatrix = [t' szo.ts_s szo.ts_d szo.ts_w1 szo.ts_w2 szo.ts_w3 szo.ts_M];

% reset the following to your preferences
outputfilename = 'ascii_output.txt'; 
dlm = ' '; % delimit output values with a space
format = '%14.7e' ; % output format, C syntax
lineend = 'pc'; % use this for Windows/DOS, where line ends are 
                    % carriage-return / newline characters
lineend = 'unix'; % use this for unix or mac, where line ends are 'newline'
                    % characters

                    % write the formatted file            
dlmwrite(outputfilename, outputmatrix, 'delimiter', dlm, ...
    'precision', format, 'newline', lineend)

disp(['Saving numerical results in ascii file ' outputfilename])

return



