function out = strainz17(in)
%
% v.3 10/22/08
strainzversion = 17.3; 
%
% ABSTRACT
%
% This routine calculates the best-fitting rigid body rotation and uniform
% strain as functions of time, and their formal errors, given
% three-component ground motion time series recorded on a seismic array.
% The theory implemented herein is presented in the papers: 
%       Spudich et al. (J. Geophys. Res., 1995), (abbreviated S95 herein) 
%       Spudich and Fletcher (Bull. Seismol. Soc. Am., 2008)  (SF08)
%       Spudich and Fletcher (Bull. Seismol. Soc. Am., 2009). (SF09)
%

% NOTE ON SPECIFYING INPUT ARRAY AND SELECTING SUBARRAYS
%
% This routine allows the user to input the coordinates and ground motion 
% time series of all stations in a seismic array having Na stations 
% and the user may select for analysis a subarray of Nplus1 <= Na stations.
%   

% DISCUSSION OF PHYSICAL UNITS OF INPUT AND OUPUT
%
% If the input seismograms are in units of displacement, the output strains
% and rotations will be in units of strain (unitless) and angle (radians). 
% If the input seismograms are in units of velocity, the output will be 
% strain rate (units = s^-1) and rotation rate (rad/s). 
% Higher temporal derivative inputs yield higher temporal derivative
% outputs. 
% 
% Input units of the array station coordinates must match the spatial units
% of the seismograms.  For example, if the input seismograms are in units
% of miles/s^2, array coordinates must be entered in miles. 

% WARNINGS
% 
% This routine does not check to verify that your array is small enough to
% conform to the assumption that the array aperture is less than 1/4 of the
% shortest seismic wavelength in the data.  See SF08 for a discussion of 
% this assumption. 

% NOTE ON COORDINATE SYSTEM
% 
% This routine assumes x1-x2-x3 is a right handed orthogonal coordinate
% system. x3 must point either up or down.  
%

% INPUT STRUCTURE: 
%
%   in  - (structure) with fields:
%           .vp - P wave speed in the soil under the array (km/s)
%           .vs - S wave speed in the soil under the array (km/s)
%               Note - vp and vs may be any unit (e.g. miles/week), and 
%               this unit need not be related to the units of the station 
%               coordinates or ground motions, but the units of vp and vs
%               must be the SAME because their ratio is used. 
%           .array_coords = a matrix of dimension Na x 3, where Na is the 
%               number of stations in the array.  
%               in.array_coords(i,j), i=1:Na, j=1:3 is the xj coordinate of 
%               station i.  Units of array_coords may be anything, but
%               see the "Discussion of input and output units" above. 
%               The origin of coordinates is arbitrary and does not affect
%               the calculated strains and rotations.  Stations may be 
%               entered in any order. 
%           .ts1 = a matrix of x1-component seismograms, dimension nt x Na 
%               in.ts1(j,k) j=1:nt, k=1:Na contains the kth time sample of 
%               the x1 component ground motion at station k. NOTE that the
%               seismogram in column k must correspond to the station whose
%               coordinates are in row k of in.array_coords. nt is the 
%               number of time samples in the seismograms.  Seismograms 
%               may be displacement, velocity, acceleration, jerk, etc. 
%               See the "Discussion of input and output units" above. 
%           .ts2 = same as ts1, but for the x2 component of motion.
%           .ts3 = same as ts1, but for the x3 (up or down) component of 
%                   motion. 
%               >>> NOTE <<< this code assumes that ts1(j,:), ts2(j,:), and
%               ts3(j,:) are all sampled SIMULTANEOUSLY. 
%           .sigmau - standard deviation (NOT VARIANCE) of ground 
%               noise, corresponds to sigma-sub-u in S95 lines above eqn (A5).
%               NOTE: This may be entered as a scalar, vector, or matrix!
%               - If sigmau is a scalar, it will be used for all
%                   components of all stations. 
%               - If sigmau is a vector of length Na, sigmau(i)
%                   will be the noise assigned to all components 
%                   of the station corresponding to array_coords(i,:)
%               - If sigmau is a matrix of dimension  Na x 3, then
%                   sigmau(i,j) is used as the noise of station i, 
%                   component j.  
%               In all cases, this routine assumes that the noise
%               covariance between different stations and/or components is
%               zero.  
%           .subarray = row vector of subarray stations to use.  I.e. if
%               subarray = [1 4 10], then only rows 1, 4, and 10 of 
%               array_coords will be used, and only ground motion time
%               series in the first, fourth, and tenth columns of ts1 will be
%               used. Nplus1 is the number of elements in the subarray
%               vector, and N is set to Nplus1 - 1. To use all stations in
%               the array, set in.subarray = (1:Na), where Na is the total
%               number of stations in the array (equal to the number of
%               rows of in.array_coords. Sequence of stations in the
%               subarray vector is unimportant; i.e. subarray = [1 4 10]
%               will yield essentially the same rotations and strains
%               as subarray = [10 4 1]. "Essentially" because permuting 
%               subarray sequence changes the d vector, yielding a slightly
%               different numerical result. 
%               

% OUTPUT STRUCTURE: (ts_ below means "time series")
%
%   out - (structure) with fields:  
%           .A = (matrix, dimension 3N x 6) - data mapping matrix 'A' of
%               S95(A4)
%           .g = (matrix, dimension 6 x 3N) - generlized inverse matrix
%               relating ptilde and data vector, in S95(A5)
%           .Ce - (4 x 4) covariance matrix of the 4 independent strain
%               tensor elements e11, e21, e22, e33 

%           .ts_d      - (vector, dimension nt x 1) -  dilatation
%               (trace of the 3x3 strain tensor) as a function of time
%           .sigmad = scalar, standard deviation of dilatation 

%           .ts_dh   - (vector, dimension nt x 1) -  horizontal
%               dilatation (also known as areal strain) (eEE+eNN) as a
%               function of time
%           .sigmadh = scalar, standard deviation of horizontal dilatation
%               (areal strain) 

%           .ts_s   - (vector, dimension nt x 1) -  maximum
%               strain ( .5*(max eigval of e - min eigval of e) as a 
%               function of time, where e is the 3x3 strain tenso
%           .Cgamma - (4 x 4) covariance matrix of the 4 independent shear 
%               strain tensor elements g11, g12, g22, g33 (includes full
%               covariance effects). gamma is traceless part of e. 

%           .ts_sh  - (vector, dimension nt x 1) -  maximum
%               horizontal strain ( .5*(max eigval of eh - min eigval of
%               eh) as a function of time, where eh is e(1:2,1:2)        
%           .Cgammah - (3 x 3) covariance matrix of the 3 independent 
%               horizontal shear strain tensor elements gamma11, gamma12,
%               gamma22 gamma is traceless part of e. 

%           .ts_wmag - (vector, dimension nt x 1) -  total rotation
%               angle (radians) as a function of time.  I.e. if the
%               rotation vector at the j'th time step is w = [w1 w2
%               w3], then ts_wmag(j) = sqrt( w'*w ); positive 
%               for right-handed rotation  
%           .Cw - (3 x 3) covariance matrix of the 3 independent
%               rotation tensor elements w21, w31, w32 

%           .ts_w1  - (vector, dimension nt x 1) -  rotation
%               (rad) about the x1 axis, positive for right-handed rotation
%           .sigmaw1 = scalar, standard deviation of the ts_w1
%               (sigma-omega-1 in SF08) 

%           .ts_w2  - (vector, dimension nt x 1) -  rotation
%               (rad) about the x2 axis, positive for right-handed rotation
%           .sigmaw2 = scalar, standard deviation of ts_w2
%               (sigma-omega-2 in SF08) 

%           .ts_w3  - (vector, dimension nt x 1) - "torsion", rotation 
%               (rad) about a vertical up or down axis, i.e. x3, positive 
%               for right-handed rotation   
%           .sigmaw3 = scalar, standard deviation of the torsion
%               (sigma-omega-3 in SF08) 

%           .ts_tilt     - (vector, dimension nt x 1) -  tilt (rad)
%               (rotation about a horizontal axis, positive for right
%               handed rotation) 
%               as a function of time.  tilt = sqrt( w1^2 + w2^2)
%           .sigmat = scalar, standard deviation of the tilt
%               (not defined in SF08, From Papoulis (1965, p. 195, example 7.8))

%           .ts_data - matrix (nt x 3N). time series of 
%               the observed displacement 
%               differences, which are the di in S95 eqn A1. 
%           .ts_pred - matrix (nt x 3N) time series of 
%               the fitted model's predicted displacement difference 
%               Note that the fitted model displacement
%               differences correspond to A*ptilde, where A is the big
%               matrix in S95 eqn A4 and ptilde is S95 eqn A5.
%           .ts_misfit - matrix (nt x 3N) time series of the residuals 
%               (fitted model 
%               displacement differences minus observed displacement 
%               differences). Note that the fitted model displacement
%               differences correspond to A*ptilde, where A is the big
%               matrix in S95 eqn A4 and ptilde is S95 eqn A5. 
%           .ts_M - (vector, dimension nt x 1) Time series of M, misfit
%               ratio of S95, p. 688. 

%           .ts_ptilde - matrix (nt x 6) - solution
%               vector p-tilde (from S95 eqn A5) as a function of time
%           .Cp - 6x6 solution covariance matrix defined in SF08. 

% DISCLAIMER: 
%
% Although this program has been used by the USGS, no warranty,
% expressed or implied, is made by the USGS or the United States Government
% or any authors as to the accuracy and functioning of the program and
% related program material nor shall the fact of distribution constitute
% any such warranty, and no responsibility is assumed by the USGS in
% connection therewith.

% COPYRIGHT INFO
% This m-file is a U.S. Government work and is not subject to copyright. It
% may be freely distributed. 

% start the code -------------------------------------------------

% transfer input data from structure to variables

subarray = in.subarray; 
vp = in.vp;
vs = in.vs;
ts1 = in.ts1;
ts2 = in.ts2;
ts3 = in.ts3; 
array_coords = in.array_coords; 
sigmau = in.sigmau; 


% This assumes that all stations and components have the same number of 
% time samples, nt
[nt,Na] = size(ts1);

% check to ensure all components have same duration
if ~isequal( size(ts1), size(ts2) )
    error('ts1 and ts2 have different sizes')
end
if ~isequal( size(ts1), size(ts3) )
    error('ts1 and ts3 have different sizes')
end

% check to verify that the number of stations in ts1 agrees with the number
% of stations in array_coords
[nrac, ncac] = size(array_coords); 
if nrac ~= Na
    error(['ts1 has ' int2str(Na) ' columns(stations) but ' ...
        'in.array_coords has ' int2str(nrac) ' rows(stations)'])
end

% check stations in subarray exist
if min(subarray) < 1
    error('In subarray you have specified a station number < 1')
end
if max(subarray) > Na
    error('In subarray you have specified a station number > Na')
end

% extract the stations of the subarray to be used
subarraycoords = array_coords(subarray,:); 

% count number of subarray stations Nplus1 and N, number of station offsets
Nplus1 = length(subarray);
N = Nplus1-1; 

if Nplus1 < 3
    error (['The problem is underdetermined for fewer than 3 stations'])
elseif Nplus1 == 3
    disp ('WARNING: For a 3-station array the problem is even-determined')
end

% ------------------- NOW SOME SEISMOLOGY!! --------------------------

% constants
eta = 1 - 2*vs^2/vp^2;
%

% FORM A MATRIX, WHICH RELATES MODEL VECTOR OF 6 DISPLACEMENT DERIVATIVES
%  TO VECTOR OF OBSERVED DISPLACEMENT DIFFERENCES.
%  DIM(A) = (3*NOFFSETS) * 6
% MODEL VECTOR IS [ u1,1 u1,2 u1,3 u2,1 u2,2 u2,3 ]'
% first initialize A to the null matrix
A = [];
%
%form station offset vectors.  Place in column 1 of junk the 
% station number, and in columns 2:4 the x-y-z offsets
junk = zeros(N,3);
z3t = [0 0 0]; % 8/26/02.I.3'
for i=1:N ;
     junk(i,1:3) = subarraycoords(i+1,1:3) - subarraycoords(1,1:3);
     ss = junk(i,1:3);
     A = [A; ss z3t; z3t ss ; ...
          -eta*ss(3) 0 -ss(1) 0 -eta*ss(3) -ss(2) ];
end
%
%------------------------------------------------------
% define data covariance matrix Cd. 
% step 1 - define data differencing matrix D
%          dimension of D is (3*N) * (3*Nplus1)
I3 = eye(3);
II = eye(3*N);
Cd = [];
D = [];
%
for i=1:N; D = [D -I3]; end;
D = [D; II]';

% step 2 - define displacement u covariance matrix Cu
% This assembles a covariance matrix Cu that reflects actual data errors.
[nrsigu, ncsigu] = size(sigmau);
minsz = min(size(sigmau)); 
maxsz = max(size(sigmau)); 

% populate Cu depending on the size of sigmau
if nrsigu*ncsigu == 1
    % sigmau is a scalar.  Make all diag elements of Cu the same
    Cu = sigmau.^2 * eye(3*Nplus1); 
elseif (minsz == 1) & (maxsz > 1)
    % sigmau is a row or column vector
    % check dimension is okay
    if maxsz ~= Na
        error(['sigmau must have ' int2str(Na) ' elements'])
    end
    sigmau = reshape(sigmau,maxsz,1); % make it a column vector
    junk = [sigmau sigmau sigmau].^2; % matrix of variances
    Cu = diag( reshape(  junk(subarray,:)', 3*Nplus1,1)    );
elseif isequal( size(sigmau), [Na 3])
    Cu = diag( reshape(  (sigmau(subarray,:).^2)', 3*Nplus1,1)    );
else
    error(['sigmau has the wrong dimensions'])
end

%
% Cd is the covariance matrix of the displ differences
%  dim(Cd) is (3*N) * (3*N)
Cd = D * Cu * D' ;

%---------------------------------------------------------
% form generalized inverse matrix g.  dim(g) is 6 x (3*N)
Cdi = inv(Cd);
AtCdiA = A' * Cdi * A; 
g = inv ( AtCdiA ) * A' * Cdi;

condition_number = cond(AtCdiA); 

if condition_number > 100
    disp('-------------------------------------------------------')
    disp(['WARNING: Condition number = ' num2str(condition_number)])
    disp('-------------------------------------------------------')
else
    disp(['In strainz17, matrix condition number = ' num2str(condition_number)])
end

%
% set up storage for vectors that will contain time series
ts_wmag = NaN * ones(nt,1);
ts_w1 = NaN * ones(nt,1);
ts_w2 = NaN * ones(nt,1);
ts_w3 = NaN * ones(nt,1);
ts_tilt = NaN * ones(nt,1);
ts_dh = NaN * ones(nt,1);
ts_dd = NaN * ones(nt,1); 
ts_sh = NaN * ones(nt,1);
ts_s = NaN * ones(nt,1);
ts_M = NaN * ones(nt,1);
ts_pred = NaN * ones(nt,3*N);
ts_misfit = NaN * ones(nt,3*N);
ts_M = NaN * ones(nt,1);
ts_data = NaN * ones(nt,3*N);
ts_ptilde = NaN * ones(nt,6);
%
% other matrices
udif = NaN * ones(3,N); 

%---------------------------------------------------------------
% here we define 4x6 Be and 3x6 Bw matrices.  these map the solution
% ptilde to strain or to rotation.  These matrices will be used 
% in the calculation of the covariances of strain and rotation.
% Columns of both matrices correspond to the model solution vector
%  containing elements [u1,1 u1,2 u1,3 u2,1 u2,2 u2,3 ]'
%
% the rows of Be correpond to e11 e21 e22 and e33
Be = zeros(4,6);
Be(1,1) = 2;
Be(2,2) = 1;
Be(2,4) = 1;
Be(3,5) = 2;
Be(4,1) = -2*eta;
Be(4,5) = -2*eta;
Be = Be * .5;
%
% the rows of Bw correspond to w21 w31 and w32
Bw = zeros(3,6);
Bw(1,2) = 1;
Bw(1,4) = -1;
Bw(2,3) = 2;
Bw(3,6) = 2;
Bw = Bw * .5;
%
%
% this is the 4x6 matrix mapping solution to total shear strain gamma
%  where gamma = strain - tr(strain)/3 * eye(3)
%  the four elements of shear are 11, 12, 22, and 33.  It is symmetric.
aa = (2+eta)/3;
b = (1-eta)/3;
c = (1+2*eta)/3;
Bgamma = zeros(4,6);
% 9/14/92.II.4
Bgamma(1,1) = aa;
Bgamma(1,5) = -b;
Bgamma(2,2) = .5;
Bgamma(2,4) = .5;
Bgamma(3,1) = -b;
Bgamma(3,5) = aa;
Bgamma(4,1) = -c;
Bgamma(4,5) = -c;
%   
% this is the 3x6 matrix mapping solution to horizontal shear strain 
%   gamma 
%  the four elements of horiz shear are 11, 12, and 22.  It is symmetric.
Bgammah = zeros(3,6);
Bgammah(1,1) = .5;
Bgammah(1,5) = -.5;
Bgammah(2,2) = .5;
Bgammah(2,4) = .5;
Bgammah(3,1) = -.5;
Bgammah(3,5) = .5;
%

% solution covariance matrix.  dim(Cp) = 6 * 6
% corresponding to solution elements [u1,1 u1,2 u1,3 u2,1 u2,2 u2,3 ]
Cp = g * Cd * g';

% calculate covariance matrices for strain and rotation elements

% Covariance of strain tensor elements 
% Ce should be 4x4, corresp to e11, e21, e22, e33
Ce = Be * Cp * Be';
% Cw should be 3x3 corresp to w21, w31, w32
Cw = Bw * Cp * Bw';
%
% Cgamma is 4x4 corresp to 11, 12, 22, and 33.  
Cgamma = Bgamma * Cp * Bgamma';
%
%  Cgammah is 3x3 corresp to 11, 12, and 22
Cgammah = Bgammah * Cp * Bgammah';
%
% covariance of the horizontal dilatation and the total dilatation
%  both are 1x1, i.e. scalars
Cdh = Cp(1,1) + 2*Cp(1,5) + Cp(5,5);
sigmadh = sqrt(Cdh);

sigmadsq = (1-eta).^2 * Cdh; % covariance of the (total) dilatation, ts_dd
sigmad = sqrt(sigmadsq);
%  
% Cw3, covariance of w3 rotation, i.e. torsion, is 1x1, i.e. scalar
Cw3 = (Cp(2,2) - 2*Cp(2,4) + Cp(4,4)) / 4; 
sigmaw3 = sqrt(Cw3);

% For tilt cannot use same approach because tilt is not a linear function
%  of the solution.  Here is an approximation : 
% For tilt use conservative estimate from 
%  Papoulis (1965, p. 195, example 7.8)
sigmaw1 = sqrt(Cp(6,6)); 
sigmaw2 = sqrt(Cp(3,3)); 
sigmat = max(sigmaw1,sigmaw2) * sqrt( 2-pi/2 ); 

%
% BEGIN LOOP OVER DATA POINTS IN TIME SERIES==============
%
for itime=1:nt
    %
    %
    % data vector is differences of stn i displ from stn 1 displ
    % sum the lengths of the displ difference vectors
    sumlen = 0;
    for i=1:N
        udif(1,i) = ts1(itime, subarray(i+1)) - ts1(itime, subarray(1));
        udif(2,i) = ts2(itime, subarray(i+1)) - ts2(itime, subarray(1));
        udif(3,i) = ts3(itime, subarray(i+1)) - ts3(itime, subarray(1));
        sumlen = sumlen + sqrt( udif(:,i)' *  udif(:,i) ) ;
    end

    data = udif(:); 
    %
    % form solution
    % ptilde is (u1,1 u1,2 u1,3 u2,1 u2,2 u2,3)'
    ptilde = g * data;% % 8/26/02.I.6 bottom
    %
    % place in uij_vector the full 9 elements of the displacement gradients
    % uij_vector is (u1,1 u1,2 u1,3 u2,1 u2,2 u2,3 u3,1 u3,2 u3,3)'
    % The following implements the free surface boundary condition 
    u31 = -ptilde(3); 
    u32 = -ptilde(6); 
    u33 = -eta * (ptilde(1) + ptilde(5));  
    uij_vector = [ ptilde ; u31 ; u32 ; u33 ];%
    %
    %
    % calculate predicted data
    pred = A * ptilde; % 9/8/92.I.3(9) and 8/26/92.I.3' bottom
    %
    % calculate  residuals (misfits concatenated for all stations)
    misfit = pred - data;
    
    % Calculate ts_M, misfit ratio.
    % calculate summed length of misfits (residual displacments)
    misfit_sq = misfit .* misfit ;
    misfit_sumsq = sum( reshape(misfit_sq,3,N) );
    misfit_len = sum( sqrt(misfit_sumsq) );
    ts_M(itime) = misfit_len / sumlen;%
    %
    ts_data(itime,1:3*N) = data';%
    ts_pred(itime,1:3*N) = pred';%
    ts_misfit(itime,1:3*N) = misfit';%
    ts_ptilde(itime,:) = ptilde';
 %
    %---------------------------------------------------------------
    %populate the displacement gradient matrix U
    U = zeros(3,3);
    U(:) = uij_vector;
    U = U';
    %
    % calculate strain tensors
    % Fung eqn 5.1 p 97 gives dui = (eij-wij)*dxj
    e = .5 * (U + U');%
 
    % Three components of the rotation vector omega (=w here)
    w(1) = -ptilde(6); 
    w(2) = ptilde(3); 
    w(3) = .5*(ptilde(4)-ptilde(2)); 

    % amount of total rotation is length of rotation vector
    ts_wmag(itime) = sqrt(w*w');%
    %
    % Calculate tilt and torsion
    ts_w1(itime) = w(1); 
    ts_w2(itime) = w(2); 
    ts_w3(itime) = w(3); % torsion in radians
    ts_tilt(itime) = sqrt( w(1)^2 + w(2)^2 ) ; % % 7/21/06.II.6(19), amount of tilt in radians
  
    %---------------------------------------------------------------
    %
    % Here I calculate horizontal quantities only
    % ts_dh is horizontal dilatation (+ --> expansion).  
    % Total dilatation, ts_dd, will be calculated outside the time step loop.
    % 
    ts_dh(itime) = e(1,1) + e(2,2);%
    %
    % find maximum shear strain in horizontal plane, and find its azimuth
    eh = [e(1,1) e(1,2); e(2,1) e(2,2)];% 7/21/06.II.2(4)
    gammah = eh - trace(eh)*eye(2)/2.;% 9/14/92.II.4, 7/21/06.II.2(5)

    % eigvecs are principal axes, eigvals are principal strains
    [eigvecs,eigvals] = eig(gammah);
    % max shear strain, from Fung (1965, p71, eqn (8)
    ts_sh(itime) = .5 * (max(diag(eigvals)) - min(diag(eigvals)) );%

    % calculate max of total shear strain, not just horizontal strain
    % eigvecs are principal axes, eigvals are principal strains
    [eigvect,eigvalt] = eig(e);
    % max shear strain, from Fung (1965, p71, eqn (8)
    ts_s(itime) = .5 * (max(diag(eigvalt)) - min(diag(eigvalt)) );%
    %
end %END OF FOR LOOP OVER ALL TIME POINTS
%==================================================================
%
% (total) dilatation is a scalar times horizontal dilatation owing to ther
% free surface boundary condition 
ts_d = ts_dh * (1-eta);
%
% load output structure
out.A = A; 

out.g = g; 

out.Ce = Ce;

out.ts_d = ts_d;
out.sigmad = sigmad; 

out.ts_dh = ts_dh;
out.sigmadh = sigmadh; 

out.ts_s = ts_s;
out.Cgamma = Cgamma;

out.ts_sh = ts_sh; 
out.Cgammah = Cgammah;

out.ts_wmag = ts_wmag;
out.Cw = Cw;

out.ts_w1 = ts_w1;
out.sigmaw1 = sigmaw1; 

out.ts_w2 = ts_w2;
out.sigmaw2 = sigmaw2; 

out.ts_w3 = ts_w3;
out.sigmaw3 = sigmaw3; 

out.ts_tilt = ts_tilt; 
out.sigmat = sigmat;

out.ts_data = ts_data; 
out.ts_pred = ts_pred;
out.ts_misfit = ts_misfit; 
out.ts_M = ts_M; 

out.ts_ptilde = ts_ptilde; 
out.Cp = Cp; 

out.ts_M = ts_M; 

out.strainzversion = strainzversion;
%